from .utils import floor, path, vector

__title__ = 'freegames'
__version__ = '1.4.3'
__build__ = 0x010403
__author__ = 'Grant Jenks'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2017 Grant Jenks'
